--------------------------------------------------------------------
-- Company       : Tallinn University of Technology
-- Engineer      : Anti Sullin
-- Creation Date : 05/02/2006
-- Tool Versions : WebPACK 8.1i
--
--  PS/2 KEYBOARD CONTROLLER
--
-- Description:
--		Interface, that listens PS/2 keyboard input and responds to keypresses.
--
-- Revision:
--    1.0.0: Initial release
--------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity keyb is
    Port ( ps2c : in  STD_LOGIC;
           ps2d : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           btns : out  STD_LOGIC_VECTOR (3 downto 0));
end keyb;

architecture Behavioral of keyb is

signal ps2rx : STD_LOGIC; -- ps2 received one byte
signal ps2rxd1 : STD_LOGIC_VECTOR(7 downto 0); -- last byte
signal ps2rxd2 : STD_LOGIC_VECTOR(7 downto 0); -- pre last byte
signal ps2rxd3 : STD_LOGIC_VECTOR(7 downto 0); -- pre pre last byte
signal ps2temp : STD_LOGIC_VECTOR(7 downto 0); -- temp register for ps2 rx

signal ps2creg : STD_LOGIC_VECTOR (15 downto 0); -- clock debounce shift reg
signal ps2cdeb : STD_LOGIC; -- filtered ps2 clock
signal ps2rxstate : integer range 0 to 11;



begin
-- PS2 clock debounce filter
process (clk) 
begin
if clk'event and clk='1' then
	ps2creg<=ps2creg(14 downto 0) & ps2c;
end if;
end process;


-- PS2 clock debounce
process (ps2creg)
begin
	if ps2creg = "1111111111111111" then
		ps2cdeb <= '0';
	end if;
	if ps2creg = "0000000000000000" then
		ps2cdeb <= '1';
	end if;
end process;


-- PS2 rx
process (ps2cdeb)
begin
	if ps2cdeb'event and ps2cdeb='0' then
		ps2rx <= '0';
		case ps2rxstate is
			when 0 => --start bit
				if ps2d='0' then 
					ps2rxstate<=ps2rxstate+1;
				end if;
			when 9 => --parity
				if '1' = (ps2d xor ps2temp(0) xor ps2temp(1) xor ps2temp(2) xor ps2temp(3) xor ps2temp(4) xor ps2temp(5) xor ps2temp(6) xor ps2temp(7)) then
					ps2rxd3<=ps2rxd2;
					ps2rxd2<=ps2rxd1;
					ps2rxd1<=ps2temp;
					ps2rx <= '1';
				end if;
				ps2rxstate <= ps2rxstate+1;
			when 10 => --stop bit
				if ps2d='1' then 
					ps2rxstate <= 0;
				end if;
			when others => -- data bits
				ps2temp <= ps2d & ps2temp(7 downto 1);
				ps2rxstate <= ps2rxstate + 1;
		end case;
	end if;
end process;

-- rx arrow keys: 0-dn, 1-up, 2-left, 3-right
process (ps2rx)
begin
if ps2rx'event and ps2rx='0' then
		if ps2rxd1=X"75" and (ps2rxd2=X"E0" xor ps2rxd3 = X"E0") then
			if ps2rxd2 = X"F0" then
				btns(0)<='0';
			else
				btns(0)<='1';
			end if;
		end if;
		
		if ps2rxd1=X"72" and (ps2rxd2=X"E0" xor ps2rxd3 = X"E0") then
			if ps2rxd2 = X"F0" then
				btns(1)<='0';
			else
				btns(1)<='1';
			end if;
		end if;
		
		if ps2rxd1=X"6B" and (ps2rxd2=X"E0" xor ps2rxd3 = X"E0") then
			if ps2rxd2 = X"F0" then
				btns(2)<='0';
			else
				btns(2)<='1';
			end if;
		end if;
		
		if ps2rxd1=X"74" and (ps2rxd2=X"E0" xor ps2rxd3 = X"E0") then
			if ps2rxd2 = X"F0" then
				btns(3)<='0';
			else
				btns(3)<='1';
			end if;
		end if;
end if;
end process;




end Behavioral;

